@@ -23,6 +23,7 @@ functions = {
     "number":       {"name": "number", "return_type": "number", "args": ["*"]},
     "text":         {"name": "text", "return_type": "text", "args": ["*"]},
     "char":         {"name": "_char_at", "return_type": "text", "args": ["text", "number"]},
+    "print_debug":  {"name": "_debug", "return_type": "null", "args": ["**"]},
     "print":        {"name": "_print", "return_type": "null", "args": ["**"]},
     "print_raw":    {"name": "_print_raw", "return_type": "null", "args": ["**"]},
     "input_number": {"name": "_input_number", "return_type": "number", "args": []},
@@ -32,6 +33,7 @@ functions = {
     "random":       {"name": "_random", "return_type": "number", "args": ["number"]},
     "key":          {"name": "_key", "return_type": "number", "args": ["text"]},
     "sleep":        {"name": "_sleep", "return_type": "null", "args": ["number"]},
+    "trace":        {"name": "_trace", "return_type": "null", "args": ["number", "number"]},
     "+":            {"name": "_add", "return_type": "number", "args": ["number", "number"]},
     "-":            {"name": "_sub", "return_type": "number", "args": ["number", "number"]},
     "++":           {"name": "_append", "return_type": "text", "args": ["text", "text"]},
@@ -39,6 +41,7 @@ functions = {
     "/":            {"name": "_div", "return_type": "number", "args": ["number", "number"]},
     "%":            {"name": "_mod", "return_type": "number", "args": ["number", "number"]},
     "^":            {"name": "_pow", "return_type": "number", "args": ["number", "number"]},
+    "abs":          {"name": "_abs", "return_type": "number", "args": ["number"]},
     "==":           {"name": "_eql_val", "return_type": "number", "args": ["*", "*"]},
     "===":          {"name": "_eql_ref", "return_type": "number", "args": ["*", "*"]},
     ">":            {"name": "_gt", "return_type": "number", "args": ["number", "number"]},
@@ -325,6 +328,9 @@ def to_python(t, state, indent, scope, namespace):
                     raise StartError(f"ERROR in line {line}: Function <{function}> misses return statement and should return type <{returnType}>.")
 
     elif t.data == "return":
+        result = f"\n{indent}StartError.lineNumber = {line}"
+        result += "\n" + indent + "check_events()"
+
         if namespace == "":
             raise StartError(f"ERROR in line {line}: Found a return statement outside of a function.")
         return_expression = to_python(get(t, "return_expression"), state, indent, scope, namespace)
@@ -333,7 +339,7 @@ def to_python(t, state, indent, scope, namespace):
             if symbol_type(line, return_symbol) != returnType:
                 raise StartError(f"ERROR in line {line}: Type mismatch in function <{namespace[0:-1]}>, should return type <{returnType}> but instead returns <{symbol_type(line, return_symbol)}>.")
             else:
-                result = f"\n{indent}return {return_expression}"
+                result += f"\n{indent}return {return_expression}"
                 found_return = True
 
         elif return_expression == "" and returnType != "void":  # This means we found no expression but we do need one, so raise an error
@@ -341,7 +347,7 @@ def to_python(t, state, indent, scope, namespace):
         elif return_expression != "" and returnType == "void":  # This means we found an expresswion but we dont need one, so raise an error
             raise StartError(f"ERROR in line {line}: Function <{namespace[0:-1]}> should not return a value as it is defined as void (no return).")
         else:  # we found only a return and we need that
-            result = f"\n{indent}return"
+            result += f"\n{indent}return"
             found_return = True
 
     elif t.data == "instruction":
@@ -365,7 +371,11 @@ def to_python(t, state, indent, scope, namespace):
                 if t.data == "variable_declaration" and scope == "type":  # I am parsing type declaration variables, and each of these need to be added to the constructor as argument types
                     functions[namespace.rsplit(".", 1)[0]]["args"].append(type_dec)
                 if type_dec not in types:  # If the type is not found yet, then make sure to check after the entire compilation that the type does exist
-                    types_to_check[variable] = (type_dec, line)
+                    if (buildingSymbolTable):
+                        types_to_check[variable] = (type_dec, line)
+                    else:
+                        raise StartError(f"ERROR in line {line}: Type <{type_dec}> not defined.")
+
 
             # If we are in the global scope (not a function of type constructor) OR this is a normal var def in a function just define it
             if scope == "" or (scope == "function" and t.data == "variable_declaration"):
@@ -378,8 +388,7 @@ def to_python(t, state, indent, scope, namespace):
     elif t.data == "statement":  # pass it on with a newline for the next statememt
         if not buildingSymbolTable and (compilerPass == 1 or (compilerPass == 0 and scope != "")):  # ignore this the first pass in the global scope to process imports and type and function defs only
             result = f"\n{indent}StartError.lineNumber = {line}"
-            if not parsing_event:
-                result += "\n" + indent + "check_events()"
+            result += "\n" + indent + "check_events()"
 
             result += "\n" + indent + to_python(t.children[0], state, indent, scope, namespace)
 
@@ -523,12 +532,20 @@ def compile_start(parse_tree):
         to_python(inst, state="", indent="", scope="", namespace="")
     compilerPass = 0
 
+    for t, l in types_to_check.values():
+        if t not in types:
+            raise StartError(f"ERROR in line {l}: Type <{t}> not defined.")
+
     print("Compiling types and functions")
     buildingSymbolTable = False
     for inst in parse_tree.children:
         result += to_python(inst, state="", indent="", scope="", namespace="")
     result += "\ntry:"
 
+    for t, l in types_to_check.values():
+        if t not in types:
+            raise StartError(f"ERROR in line {l}: Type <{t}> not defined.")
+
     print("Compiling main")
     compilerPass = 1
     for inst in parse_tree.children:
@@ -597,7 +614,8 @@ def main(file_name):
             import_start = "from import_start import *"
         else:
             import_start = "from start_compiler.import_start import *"
-        outfile = Path(file_name).stem  # Removes both path and extension    
+        # outfile = Path(file_name).stem  # Removes both path and extension
+        outfile = str(Path(file_name).with_suffix(""))  # Removes extension
         f = open(outfile + ".py", "w")
 
         # replace tabs with space to make the whitespace consistent
