@@ -15,9 +15,16 @@ checkingEvents = False
 
 
 def check_events():
+    if StartError.trace_line_print:
+        print(f"linenumber: {StartError.lineNumber}")
+    if StartError.trace_count:
+        StartError.trace_counter += 1
+
     global checkingEvents
+    # prevent checking event during events
     if checkingEvents:
         return
+
     checkingEvents = True
     for e in start_events.values():
         e()
@@ -153,6 +160,10 @@ def _pow(n1, n2):
     null_check(n1, n2)
     return number(n1.value ** n2.value)
 
+def _abs(n1):
+    null_check(n1)
+    return number(abs(n1.value))
+
 def _eql_val(n1, n2):
     # null_check(n1,n2)
     return number(n2.toStr() == n1.toStr())
@@ -204,7 +215,8 @@ def add_key(key):
 
 def remove_key(key):
     if isinstance(key, keyboard.KeyCode):
-        pressed_keys.clear()
+        #pressed_keys.clear()
+        pressed_keys.remove(key.char)
 
 listener = keyboard.Listener(on_press=add_key, on_release=remove_key)
 listener.daemon = True
@@ -228,7 +240,7 @@ __all__ = [
  "_set", "_get", "_do_get", "_null", "_char_at",
  "_print_raw", "_print", "_input_number", "_input_text",
  "_len", "_text_len", "_random", "_add", "_sub", "_mul",
- "_div", "_mod", "_pow", "_eql_val", "_eql_ref",
+ "_div", "_mod", "_pow", "_abs", "_eql_val", "_eql_ref",
  "_gt", "_lt", "_gte", "_lte", "_and", "_or", "_not",
  "_append", "_key", "_sleep"
 ]
